@@ -1,9 +1,7 @@
-from __future__ import unicode_literals
-
 import copy
 
 
-class Choices(object):
+class Choices:
     """
     A class to encapsulate handy functionality for lists of choices
     for a Django model field.
@@ -57,7 +55,6 @@ class Choices(object):
 
         self._process(choices)
 
-
     def _store(self, triple, triple_collector, double_collector):
         self._identifier_map[triple[1]] = triple[0]
         self._display_map[triple[0]] = triple[2]
@@ -65,7 +62,6 @@ class Choices(object):
         triple_collector.append(triple)
         double_collector.append((triple[0], triple[2]))
 
-
     def _process(self, choices, triple_collector=None, double_collector=None):
         if triple_collector is None:
             triple_collector = self._triples
@@ -94,18 +90,18 @@ class Choices(object):
                     raise ValueError(
                         "Choices can't take a list of length %s, only 2 or 3"
                         % len(choice)
-                        )
+                    )
             else:
                 store((choice, choice, choice))
 
-
     def __len__(self):
         return len(self._doubles)
 
-
     def __iter__(self):
         return iter(self._doubles)
 
+    def __reversed__(self):
+        return reversed(self._doubles)
 
     def __getattr__(self, attname):
         try:
@@ -113,11 +109,9 @@ class Choices(object):
         except KeyError:
             raise AttributeError(attname)
 
-
     def __getitem__(self, key):
         return self._display_map[key]
 
-
     def __add__(self, other):
         if isinstance(other, self.__class__):
             other = other._triples
@@ -125,29 +119,38 @@ class Choices(object):
             other = list(other)
         return Choices(*(self._triples + other))
 
-
     def __radd__(self, other):
         # radd is never called for matching types, so we don't check here
         other = list(other)
         return Choices(*(other + self._triples))
 
-
     def __eq__(self, other):
         if isinstance(other, self.__class__):
             return self._triples == other._triples
         return False
 
-
     def __repr__(self):
-        return '%s(%s)' % (
+        return '{}({})'.format(
             self.__class__.__name__,
-            ', '.join(("%s" % repr(i) for i in self._triples))
-            )
-
+            ', '.join("%s" % repr(i) for i in self._triples)
+        )
 
     def __contains__(self, item):
         return item in self._db_values
 
-
     def __deepcopy__(self, memo):
         return self.__class__(*copy.deepcopy(self._triples, memo))
+
+    def subset(self, *new_identifiers):
+        identifiers = set(self._identifier_map.keys())
+
+        if not identifiers.issuperset(new_identifiers):
+            raise ValueError(
+                'The following identifiers are not present: %s' %
+                identifiers.symmetric_difference(new_identifiers),
+            )
+
+        return self.__class__(*[
+            choice for choice in self._triples
+            if choice[1] in new_identifiers
+        ])
