@@ -3,10 +3,43 @@ from functools import wraps
 
 from django.core.exceptions import FieldError
 from django.db import models
-from django.db.models.fields.files import FileDescriptor
+from django.db.models.fields.files import FieldFile, FileDescriptor
 from django.db.models.query_utils import DeferredAttribute
 
 
+class LightStateFieldFile(FieldFile):
+    """
+    FieldFile subclass with the only aim to remove the instance from the state.
+
+    The change introduced in Django 3.1 on FieldFile subclasses results in pickling the
+    whole instance for every field tracked.
+    As this is done on the initialization of objects, a simple queryset evaluation on
+    Django 3.1+ can make the app unusable, as CPU and memory usage gets easily
+    multiplied by magnitudes.
+    """
+    def __getstate__(self):
+        """
+        We don't need to deepcopy the instance, so nullify if provided.
+        """
+        state = super().__getstate__()
+        if 'instance' in state:
+            state['instance'] = None
+        return state
+
+
+def lightweight_deepcopy(value):
+    """
+    Use our lightweight class to avoid copying the instance on a FieldFile deepcopy.
+    """
+    if isinstance(value, FieldFile):
+        value = LightStateFieldFile(
+            instance=value.instance,
+            field=value.field,
+            name=value.name,
+        )
+    return deepcopy(value)
+
+
 class DescriptorMixin:
     tracker_instance = None
 
@@ -20,7 +53,7 @@ class DescriptorMixin:
             was_deferred = True
         value = super().__get__(instance, owner)
         if was_deferred:
-            self.tracker_instance.saved_data[field_name] = deepcopy(value)
+            self.tracker_instance.saved_data[field_name] = lightweight_deepcopy(value)
         return value
 
     def _get_field_name(self):
@@ -44,7 +77,7 @@ class DescriptorWrapper:
             value = self.descriptor
         if was_deferred:
             tracker_instance = getattr(instance, self.tracker_attname)
-            tracker_instance.saved_data[self.field_name] = deepcopy(value)
+            tracker_instance.saved_data[self.field_name] = lightweight_deepcopy(value)
         return value
 
     def __set__(self, instance, value):
@@ -86,11 +119,86 @@ class FullDescriptorWrapper(DescriptorWrapper):
         self.descriptor.__delete__(obj)
 
 
+class FieldsContext:
+    """
+    A context manager for tracking nested reset fields contexts.
+
+    If tracked fields is mentioned in more than one FieldsContext, it's state
+    is being reset on exiting last context that mentions that field.
+
+    >>> with fields_context(obj.tracker, 'f1', state=state):
+    ...     with fields_context(obj.tracker, 'f1', 'f2', state=state):
+    ...         obj.do_something_useful()
+    ...     # f2 is reset after inner context exit
+    ...     obj.do_something_else()
+    ... # f1 is reset after outer context exit
+    >>>
+
+    * Note that fields are counted by passing same state dict
+    * FieldsContext is instantiated using FieldInstanceTracker (`obj.tracker`)
+    * Different objects has own state stack
+
+    """
+
+    def __init__(self, tracker, *fields, state=None):
+        """
+        :param tracker: FieldInstanceTracker instance to be reset after
+            context exit
+        :param fields: a list of field names to be tracked in current context
+        :param state: shared state dict used to count number of field
+            occurrences in context stack.
+
+        On context enter each field mentioned in `fields` has +1 in shared
+        state, and on exit it receives -1. Fields that have zero after context
+        exit are reset in tracker instance.
+        """
+        if state is None:
+            state = {}
+        self.tracker = tracker
+        self.fields = fields
+        self.state = state
+
+    def __enter__(self):
+        """
+        Increments tracked fields occurrences count in shared state.
+        """
+        for f in self.fields:
+            self.state.setdefault(f, 0)
+            self.state[f] += 1
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """
+        Decrements tracked fields occurrences count in shared state.
+
+        If any field has no more occurrences in shared state, this field is
+        being reset by tracker.
+        """
+        reset_fields = []
+        for f in self.fields:
+            self.state[f] -= 1
+            if self.state[f] == 0:
+                reset_fields.append(f)
+                del self.state[f]
+        if reset_fields:
+            self.tracker.set_saved_fields(fields=reset_fields)
+
+
 class FieldInstanceTracker:
     def __init__(self, instance, fields, field_map):
         self.instance = instance
         self.fields = fields
         self.field_map = field_map
+        self.context = FieldsContext(self, *self.fields)
+
+    def __enter__(self):
+        return self.context.__enter__()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return self.context.__exit__(exc_type, exc_val, exc_tb)
+
+    def __call__(self, *fields):
+        return FieldsContext(self, *fields, state=self.context.state)
 
     @property
     def deferred_fields(self):
@@ -109,7 +217,7 @@ class FieldInstanceTracker:
 
         # preventing mutable fields side effects
         for field, field_value in self.saved_data.items():
-            self.saved_data[field] = deepcopy(field_value)
+            self.saved_data[field] = lightweight_deepcopy(field_value)
 
     def current(self, fields=None):
         """Returns dict of current values for all tracked fields"""
@@ -150,7 +258,7 @@ class FieldInstanceTracker:
             else:
                 current_value = self.get_field_value(field)
                 self.instance.refresh_from_db(fields=[field])
-                self.saved_data[field] = deepcopy(self.get_field_value(field))
+                self.saved_data[field] = lightweight_deepcopy(self.get_field_value(field))
                 setattr(self.instance, self.field_map[field], current_value)
 
         return self.saved_data.get(field)
@@ -195,6 +303,20 @@ class FieldTracker:
     def __init__(self, fields=None):
         self.fields = fields
 
+    def __call__(self, func=None, fields=None):
+        def decorator(f):
+            @wraps(f)
+            def inner(obj, *args, **kwargs):
+                tracker = getattr(obj, self.attname)
+                field_list = tracker.fields if fields is None else fields
+                with tracker(*field_list):
+                    return f(obj, *args, **kwargs)
+
+            return inner
+        if func is None:
+            return decorator
+        return decorator(func)
+
     def get_field_map(self, cls):
         """Returns dict mapping fields names to model attribute names"""
         field_map = {field: field for field in self.fields}
@@ -240,21 +362,17 @@ class FieldTracker:
 
         @wraps(original)
         def inner(instance, *args, **kwargs):
-            ret = original(instance, *args, **kwargs)
             update_fields = kwargs.get(fields_kwarg)
-            if not update_fields and update_fields is not None:  # () or []
-                fields = update_fields
-            elif update_fields is None:
-                fields = None
+            if update_fields is None:
+                fields = self.fields
             else:
                 fields = (
                     field for field in update_fields if
                     field in self.fields
                 )
-            getattr(instance, self.attname).set_saved_fields(
-                fields=fields
-            )
-            return ret
+            tracker = getattr(instance, self.attname)
+            with tracker(*fields):
+                return original(instance, *args, **kwargs)
 
         setattr(model, method, inner)
 
