@@ -1,6 +1,4 @@
-from __future__ import unicode_literals, absolute_import
-
-from model_utils.managers import SoftDeletableQuerySet, SoftDeletableManager
+from model_utils.managers import SoftDeletableManager, SoftDeletableQuerySet
 
 
 class CustomSoftDeleteQuerySet(SoftDeletableQuerySet):
