@@ -1,12 +1,12 @@
-from __future__ import unicode_literals
-
 from django.test import TestCase
 
 from model_utils.fields import StatusField
 from tests.models import (
-    Article, StatusFieldDefaultFilled, StatusFieldDefaultNotFilled,
+    Article,
     StatusFieldChoicesName,
-    )
+    StatusFieldDefaultFilled,
+    StatusFieldDefaultNotFilled,
+)
 
 
 class StatusFieldTests(TestCase):
