@@ -1,16 +1,17 @@
-from __future__ import unicode_literals
-
-from unittest import skipUnless
-
-import django
 from django.db import models
 from django.test import TestCase
 
-from tests.models import (InheritanceManagerTestRelated, InheritanceManagerTestGrandChild1,
-                                      InheritanceManagerTestGrandChild1_2, InheritanceManagerTestParent,
-                                      InheritanceManagerTestChild1,
-                                      InheritanceManagerTestChild2, TimeFrame, InheritanceManagerTestChild3
-                                      )
+from tests.models import (
+    InheritanceManagerTestChild1,
+    InheritanceManagerTestChild2,
+    InheritanceManagerTestChild3,
+    InheritanceManagerTestChild4,
+    InheritanceManagerTestGrandChild1,
+    InheritanceManagerTestGrandChild1_2,
+    InheritanceManagerTestParent,
+    InheritanceManagerTestRelated,
+    TimeFrame,
+)
 
 
 class InheritanceManagerTests(TestCase):
@@ -25,16 +26,16 @@ class InheritanceManagerTests(TestCase):
         return InheritanceManagerTestParent.objects
 
     def test_normal(self):
-        children = set([
+        children = {
             InheritanceManagerTestParent(pk=self.child1.pk),
             InheritanceManagerTestParent(pk=self.child2.pk),
             InheritanceManagerTestParent(pk=self.grandchild1.pk),
             InheritanceManagerTestParent(pk=self.grandchild1_2.pk),
-        ])
+        }
         self.assertEqual(set(self.get_manager().all()), children)
 
     def test_select_all_subclasses(self):
-        children = set([self.child1, self.child2])
+        children = {self.child1, self.child2}
         children.add(self.grandchild1)
         children.add(self.grandchild1_2)
         self.assertEqual(
@@ -47,16 +48,16 @@ class InheritanceManagerTests(TestCase):
         raise an AttributeError further in.
         """
         regex = '^.+? is not in the discovered subclasses, tried:.+$'
-        with self.assertRaisesRegexp(ValueError, regex):
+        with self.assertRaisesRegex(ValueError, regex):
             self.get_manager().select_subclasses('user')
 
     def test_select_specific_subclasses(self):
-        children = set([
+        children = {
             self.child1,
             InheritanceManagerTestParent(pk=self.child2.pk),
             InheritanceManagerTestChild1(pk=self.grandchild1.pk),
             InheritanceManagerTestChild1(pk=self.grandchild1_2.pk),
-        ])
+        }
         self.assertEqual(
             set(
                 self.get_manager().select_subclasses(
@@ -66,12 +67,12 @@ class InheritanceManagerTests(TestCase):
         )
 
     def test_select_specific_grandchildren(self):
-        children = set([
+        children = {
             InheritanceManagerTestParent(pk=self.child1.pk),
             InheritanceManagerTestParent(pk=self.child2.pk),
             self.grandchild1,
             InheritanceManagerTestParent(pk=self.grandchild1_2.pk),
-        ])
+        }
         self.assertEqual(
             set(
                 self.get_manager().select_subclasses(
@@ -82,12 +83,12 @@ class InheritanceManagerTests(TestCase):
         )
 
     def test_children_and_grandchildren(self):
-        children = set([
+        children = {
             self.child1,
             InheritanceManagerTestParent(pk=self.child2.pk),
             self.grandchild1,
             InheritanceManagerTestChild1(pk=self.grandchild1_2.pk),
-        ])
+        }
         self.assertEqual(
             set(
                 self.get_manager().select_subclasses(
@@ -115,9 +116,6 @@ class InheritanceManagerTests(TestCase):
                 "inheritancemanagertestchild2").get(pk=self.child1.pk)
             obj.inheritancemanagertestchild1
 
-    def test_version_determining_any_depth(self):
-        self.assertIsNone(self.get_manager().all()._get_maximum_depth())
-
     def test_manually_specifying_parent_fk_including_grandchildren(self):
         """
         given a Model which inherits from another Model, but also declares
@@ -125,10 +123,16 @@ class InheritanceManagerTests(TestCase):
         ensure that the relation names and subclasses are obtained correctly.
         """
         child3 = InheritanceManagerTestChild3.objects.create()
-        results = InheritanceManagerTestParent.objects.all().select_subclasses()
+        qs = InheritanceManagerTestParent.objects.all()
+        results = qs.select_subclasses().order_by('pk')
 
-        expected_objs = [self.child1, self.child2, self.grandchild1,
-                         self.grandchild1_2, child3]
+        expected_objs = [
+            self.child1,
+            self.child2,
+            self.grandchild1,
+            self.grandchild1_2,
+            child3
+        ]
         self.assertEqual(list(results), expected_objs)
 
         expected_related_names = [
@@ -137,6 +141,7 @@ class InheritanceManagerTests(TestCase):
             'inheritancemanagertestchild1',
             'inheritancemanagertestchild2',
             'manual_onetoone',  # this was set via parent_link & related_name
+            'child4_onetoone',
         ]
         self.assertEqual(set(results.subclasses),
                          set(expected_related_names))
@@ -148,7 +153,8 @@ class InheritanceManagerTests(TestCase):
         """
         related_name = 'manual_onetoone'
         child3 = InheritanceManagerTestChild3.objects.create()
-        results = InheritanceManagerTestParent.objects.all().select_subclasses(related_name)
+        qs = InheritanceManagerTestParent.objects.all()
+        results = qs.select_subclasses(related_name).order_by('pk')
 
         expected_objs = [InheritanceManagerTestParent(pk=self.child1.pk),
                          InheritanceManagerTestParent(pk=self.child2.pk),
@@ -164,15 +170,12 @@ class InheritanceManagerTests(TestCase):
         queryset = InheritanceManagerTestChild1.objects.values('id').filter(pk=self.child1.pk)
         self.assertEqual(list(queryset), [{'id': self.child1.pk}])
 
-    @skipUnless(django.VERSION >= (1, 9, 0), "test only applies to Django 1.9+")
-    def test_dj19_values_list_on_select_subclasses(self):
+    def test_values_list_on_select_subclasses(self):
         """
         Using `select_subclasses` in conjunction with `values_list()` raised an
         exception in `_get_sub_obj_recurse()` because the result of `values_list()`
         is either a `tuple` or primitive objects if `flat=True` is specified,
         because no type checking was done prior to fetching child nodes.
-
-        Django versions below 1.9 are not affected by this bug.
         """
 
         # Querysets are cast to lists to force immediate evaluation.
@@ -180,27 +183,26 @@ class InheritanceManagerTests(TestCase):
 
         # No argument to select_subclasses
         objs_1 = list(
-            self.get_manager().
-                select_subclasses().
-                values_list('id')
+            self.get_manager()
+                .select_subclasses()
+                .values_list('id')
         )
 
         # String argument to select_subclasses
         objs_2 = list(
-            self.get_manager().
-                select_subclasses(
+            self.get_manager()
+            .select_subclasses(
                 "inheritancemanagertestchild2"
-            ).
-                values_list('id')
+            )
+            .values_list('id')
         )
 
         # String argument to select_subclasses
         objs_3 = list(
-            self.get_manager().
-                select_subclasses(
+            self.get_manager()
+            .select_subclasses(
                 InheritanceManagerTestChild2
-            ).
-                values_list('id')
+            ).values_list('id')
         )
 
         assert all((
@@ -254,7 +256,7 @@ class InheritanceManagerUsingModelsTests(TestCase):
         objs = InheritanceManagerTestParent.objects.select_subclasses().order_by('pk')
         objsmodels = InheritanceManagerTestParent.objects.select_subclasses(
             InheritanceManagerTestChild1, InheritanceManagerTestChild2,
-            InheritanceManagerTestChild3,
+            InheritanceManagerTestChild3, InheritanceManagerTestChild4,
             InheritanceManagerTestGrandChild1,
             InheritanceManagerTestGrandChild1_2).order_by('pk')
         self.assertEqual(set(objs.subclasses), set(objsmodels.subclasses))
@@ -276,6 +278,7 @@ class InheritanceManagerUsingModelsTests(TestCase):
         models = (InheritanceManagerTestChild1,
                   InheritanceManagerTestChild2,
                   InheritanceManagerTestChild3,
+                  InheritanceManagerTestChild4,
                   InheritanceManagerTestGrandChild1,
                   InheritanceManagerTestGrandChild1_2)
 
@@ -307,7 +310,7 @@ class InheritanceManagerUsingModelsTests(TestCase):
         Confirming that giving a stupid model doesn't work.
         """
         regex = '^.+? is not a subclass of .+$'
-        with self.assertRaisesRegexp(ValueError, regex):
+        with self.assertRaisesRegex(ValueError, regex):
             InheritanceManagerTestParent.objects.select_subclasses(
                 TimeFrame).order_by('pk')
 
@@ -392,14 +395,16 @@ class InheritanceManagerUsingModelsTests(TestCase):
         """
         child3 = InheritanceManagerTestChild3.objects.create()
         results = InheritanceManagerTestParent.objects.all().select_subclasses(
-            InheritanceManagerTestChild3)
+            InheritanceManagerTestChild3).order_by('pk')
 
-        expected_objs = [InheritanceManagerTestParent(pk=self.parent1.pk),
-                         InheritanceManagerTestParent(pk=self.child1.pk),
-                         InheritanceManagerTestParent(pk=self.child2.pk),
-                         InheritanceManagerTestParent(pk=self.grandchild1.pk),
-                         InheritanceManagerTestParent(pk=self.grandchild1_2.pk),
-                         child3]
+        expected_objs = [
+            InheritanceManagerTestParent(pk=self.parent1.pk),
+            InheritanceManagerTestParent(pk=self.child1.pk),
+            InheritanceManagerTestParent(pk=self.child2.pk),
+            InheritanceManagerTestParent(pk=self.grandchild1.pk),
+            InheritanceManagerTestParent(pk=self.grandchild1_2.pk),
+            child3
+        ]
         self.assertEqual(list(results), expected_objs)
 
         expected_related_names = ['manual_onetoone']
@@ -415,6 +420,57 @@ class InheritanceManagerUsingModelsTests(TestCase):
         )
         self.assertTrue(all(result.foo == (result.id + 1) for result in results))
 
+    def test_limit_to_specific_subclass(self):
+        child3 = InheritanceManagerTestChild3.objects.create()
+        results = InheritanceManagerTestParent.objects.instance_of(InheritanceManagerTestChild3)
+
+        self.assertEqual([child3], list(results))
+
+    def test_limit_to_specific_grandchild_class(self):
+        grandchild1 = InheritanceManagerTestGrandChild1.objects.get()
+        results = InheritanceManagerTestParent.objects.instance_of(InheritanceManagerTestGrandChild1)
+
+        self.assertEqual([grandchild1], list(results))
+
+    def test_limit_to_child_fetches_grandchildren_as_child_class(self):
+        # Not sure if this is the desired behaviour...?
+        children = InheritanceManagerTestChild1.objects.all()
+
+        results = InheritanceManagerTestParent.objects.instance_of(InheritanceManagerTestChild1)
+
+        self.assertEqual(set(children), set(results))
+
+    def test_can_fetch_limited_class_grandchildren(self):
+        # Not sure if this is the desired behaviour...?
+        children = InheritanceManagerTestChild1.objects.select_subclasses()
+
+        results = InheritanceManagerTestParent.objects.instance_of(InheritanceManagerTestChild1).select_subclasses()
+
+        self.assertEqual(set(children), set(results))
+
+    def test_selecting_multiple_instance_classes(self):
+        child3 = InheritanceManagerTestChild3.objects.create()
+        children1 = InheritanceManagerTestChild1.objects.all()
+
+        results = InheritanceManagerTestParent.objects.instance_of(InheritanceManagerTestChild3, InheritanceManagerTestChild1)
+
+        self.assertEqual(set([child3] + list(children1)), set(results))
+
+    def test_selecting_multiple_instance_classes_including_grandchildren(self):
+        child3 = InheritanceManagerTestChild3.objects.create()
+        grandchild1 = InheritanceManagerTestGrandChild1.objects.get()
+
+        results = InheritanceManagerTestParent.objects.instance_of(InheritanceManagerTestChild3, InheritanceManagerTestGrandChild1).select_subclasses()
+
+        self.assertEqual({child3, grandchild1}, set(results))
+
+    def test_select_subclasses_interaction_with_instance_of(self):
+        child3 = InheritanceManagerTestChild3.objects.create()
+
+        results = InheritanceManagerTestParent.objects.select_subclasses(InheritanceManagerTestChild1).instance_of(InheritanceManagerTestChild3)
+
+        self.assertEqual({child3}, set(results))
+
 
 class InheritanceManagerRelatedTests(InheritanceManagerTests):
     def setUp(self):
@@ -435,6 +491,14 @@ class InheritanceManagerRelatedTests(InheritanceManagerTests):
                 id=self.child1.id),
             self.child1)
 
+    def test_get_method_with_select_subclasses_check_for_useless_join(self):
+        child4 = InheritanceManagerTestChild4.objects.create(related=self.related, other_onetoone=self.child1)
+        self.assertEqual(
+            str(InheritanceManagerTestChild4.objects.select_subclasses().filter(
+                id=child4.id).query),
+            str(InheritanceManagerTestChild4.objects.select_subclasses().select_related(None).filter(
+                id=child4.id).query))
+
     def test_annotate_with_select_subclasses(self):
         qs = InheritanceManagerTestParent.objects.select_subclasses().annotate(
             models.Count('id'))
@@ -454,3 +518,7 @@ class InheritanceManagerRelatedTests(InheritanceManagerTests):
         qs = InheritanceManagerTestParent.objects.annotate(
             test_count=models.Count('id')).select_subclasses()
         self.assertEqual(qs.get(id=self.child1.id).test_count, 1)
+
+    def test_clone_when_inheritance_queryset_selects_subclasses_should_clone_them_too(self):
+        qs = InheritanceManagerTestParent.objects.select_subclasses()
+        self.assertEqual(qs.subclasses, qs._clone().subclasses)
